###########################################################################
#  Vintel - Visual Intel Chat Analyzer                                    #
#  Copyright (C) 2014-15 Sebastian Meyer (sparrow.242.de+eve@gmail.com )  #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#  GNU General Public License for more details.                           #
#                                                                         #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
###########################################################################

from PyQt4.QtCore import QThread
from PyQt4.QtCore import SIGNAL

from Queue import Queue
import time

from vi import evegate
from vi import koschecker
from vi.cache.cache import Cache

from vi.resources import resource_path

class AvatarFindThread(QThread):

    def __init__(self):
        QThread.__init__(self)
        self.q = Queue()

    def add_chatentry(self, chatentry, clear_cache=False):
        if clear_cache:
            cache = Cache()
            cache.remove_avatar(chatentry.message.user)
        self.q.put(chatentry)

    def run(self):
        cache = Cache()
        last_call = 0
        wait = 300 # time between 2 requests in ms
        while True:
            chatentry = self.q.get()
            charname = chatentry.message.user
            avatar = None
            if charname == "VINTEL":
                with open(resource_path("vi/ui/res/logo_small.png"), "rb") as f:
                    avatar = f.read()
            if not avatar:
                avatar = cache.get_avatar(charname)
            if not avatar:
                diff_last_call = time.time() - last_call
                if diff_last_call < wait:
                    time.sleep((wait - diff_last_call) /1000.0)
                avatar = evegate.get_avatar_for_player(charname)
                last_call = time.time()
                if avatar:
                    cache.put_avatar(charname, avatar)
            if avatar:
                self.emit(SIGNAL("avatar_update"), chatentry, avatar)
            """
            # deactivated instand retry to load the avatar to prevent any
            # spamming. If it fails it fails. That's destiny ;)
            else:
                self.q.put(chatentry)
            """


class PlayerFindThread(QThread):
    
    def __init__(self):
        QThread.__init__(self)
        self.q = Queue()
        
    def add_chatentry(self, chatentry):
        self.q.put(chatentry)
        
    def run(self):
        cache = Cache()
        while True:
            chatentry = self.q.get()
            text = chatentry.message.utext
            parts = text.split("  ")  # split@double-space (not in name, right?)



class KOSCheckerThread(QThread):
    
    def __init__(self):
        QThread.__init__(self)
        self.q = Queue()
        
    def add_request(self, names, request_type, only_kos = False):
        self.q.put((names, request_type, only_kos))
        
    def run(self):
        while True:
            names, request_type, only_kos = self.q.get()
            has_kos = False
            try:
                state = "ok"
                check_result = koschecker.check(names)
                text = koschecker.result_to_text(check_result, only_kos)
                for name, data in check_result.items():
                    if data["kos"] in (koschecker.KOS, koschecker.RED_BY_LAST):
                        has_kos = True
                        break
            except Exception as e:
                state = "error"
                text = unicode(e)
            self.emit(SIGNAL("kos_result"), state, text, request_type, has_kos)

class MapStatisticsThread(QThread):
    
    def __init__(self):
        QThread.__init__(self)
        
    def run(self):
        try:
            statistics = evegate.get_system_statistics()
            time.sleep(5)  # sleeping to prevent a "need 2 arguments"-error
            retdata = {"result": "ok", "statistics": statistics}
            self.emit(SIGNAL("statistic_data_update"), retdata)
        except Exception as e:
            retdata = {"result": "error", "text": unicode(e)}
            self.emit(SIGNAL("statistic_data_update"), retdata)